#============================================================================
#  Name:
#    $(TARGET).MAK
#
#  Description:
#    Makefile to build the $(TARGET) downloadable module.
#
#   The following nmake targets are available in this makefile:
#
#     all           - make .elf and .mod image files (default)
#     clean         - delete object directory and image files
#     filename.o    - make object file
#
#   The above targets can be made with the following command:
#
#     nmake /f $(TARGET).mak [target]
#
#  Assumptions:
#    1. nmake requires you to have GCC cross compiler installed on your machine.
#    2. cygwin(to be precise cygwin1.dll) is installed on your machine and 
#       is in the path.  
#    3. GCCHOME environment variable points to location of GCC cross compiler
#    4. GCCLIBPATH environment variable points to location of libgcc.a
#    5. BREWADDINS environment variable points to location of BREW Addins
#    6. BREWDIR environment variable points to location of BREW SDK
#    7. The PATH environment variable contains location of GCC (GCCHOME), (GCCHOME)\arm-elf\bin,
#       and (GCCHOME)\lib\gcc-lib\arm-elf\2.95.3 or whichever version of GCC is installed.
#
#  Notes:
#    1. While building the application using this make file, the following warnings may be 
#       received. They can be safely ignored:
#		"Warning: _divsi3.o does not support interworking, whereas <Your App>.elf does not"
#		"Warning: _dvmd_tls.o does not support interworking, whereas <Your App>.elf does not"
#		"Warning: cannot find entry symbol _start; defaulting to 00008000"
#  
#============================================================================
#============================================================================
SF_HOME        = $(SFHOME)
BREW_HOME      = $(BREWDIR)
BREW_ADDINS    = $(BREWADDINS)
GCC_HOME       = $(GCCHOME)
GCC_LIBPATH    = $(GCC_HOME)\lib\gcc\arm-elf\4.1.1
TARGET         =yhelloworld
OBJS           =yhelloworld.o 
APP_INCLUDES   = -I "$(BREWDIR)\inc" -I "$(SF_HOME)\Include"

#-------------------------------------------------------------------------------
# Target file name and type definitions
#-------------------------------------------------------------------------------

EXETYPE	=	elf                # Target image file format
BINARY  =	bin
MODULE	=	mod                # Downloadable module extension

#-----------------------------------------------------------------------
# Target compile time symbol definitions
#
# Tells the SDK source stuffs that we're building a dynamic app.
#-----------------------------------------------------------------------

DYNAPP = 	-DDYNAMIC_APP

#-----------------------------------------------------------------------
# Software tool and environment definitions
#   (PREFIX needs to be set to the location of your gcc cross-compiler
#    and binutils.  STUBTOOL needs to be the path and filename of your
#    copy of makestub)
#-----------------------------------------------------------------------

GCC		   = $(GCC_HOME)\bin\arm-elf-gcc
LD		   = $(GCC_HOME)\bin\arm-elf-ld
DUMPTOOL   = $(GCC_HOME)\bin\arm-elf-objdump
ELF2MODTOOLPATH = $(ELF2MOD_TOOL_PATH)
ELF2MODTOOL	= "$(ELF2MODTOOLPATH)\elf2mod.exe"
LINKERSCRIPT = "$(SF_HOME)\Utility\GNU\LinkerScript\elf2mod_sf.x"

#-----------------------------------------------------------------------
# Compiler optimization options
#   -O0 disables compiler optimizations.  Other options probably work as
#     well.  Set to taste.
#-----------------------------------------------------------------------
OPT	=	-O2

#-----------------------------------------------------------------------
# Compiler code generation options
#   Add $(TARG) to the CODE line if you're building a Thumb binary (at
#   the moment, this doesn't work).
#-----------------------------------------------------------------------
END		=	-mlittle-endian
TARG	=	#-mthumb
CODE	=	$(END) $(TARG) -fno-builtin -fno-nonansi-builtins -fno-exceptions -fno-enforce-eh-specs -fno-unwind-tables -fno-asynchronous-unwind-tables -fno-rtti -fno-threadsafe-statics -fno-stack-check -fshort-enums -ffunction-sections -fdata-sections

#-----------------------------------------------------------------------
# Include file search path options
#   (change this to point to where the BREW SDK headers are located)
#-----------------------------------------------------------------------
INC	=	$(APP_INCLUDES)

#-----------------------------------------------------------------------
# Library search path options
#   (You can change this to point to where your interworking version of 
#    libgcc.a resides)
#-----------------------------------------------------------------------
LIBDIRS = -L"$(GCC_LIBPATH)\interwork" -L"$(SF_HOME)\Library"

#-----------------------------------------------------------------------
# Nothing below here (except for the dependencies at the bottom of the
#  file) should need to be changed for a reasonably normal compilation.
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
# Processor architecture options
# Sets the designated target processor for this applet.
# Currently, all BREW phones use the ARM 7t chip
#-----------------------------------------------------------------------
CPU	=	-mcpu=arm7tdmi

#-----------------------------------------------------------------------
# ARM Procedure Call Standard (APCS) options
# -fPIC             sets posititon independent code.  Other option: -fpic
# -mthumb-interwork enables switching between ARM and Thumb code
# -mapcs-frame      runs on systems with the frame ptr. specified in the
# 			APCS
#-----------------------------------------------------------------------
ROPI	=	
INTRWK	=	-mthumb-interwork
APCS	=	-mapcs-frame $(ROPI) $(INTRWK)

#-----------------------------------------------------------------------
# Compiler output options
# -c sets object file output only
#-----------------------------------------------------------------------
OUT	=	-c

#-----------------------------------------------------------------------
# Compiler/assembler debug Options
#   -g is the standard flag to leave debugging information in the
#   object files.
#-----------------------------------------------------------------------
DBG	=

#-----------------------------------------------------------------------
# Linker Options
# -o sets the output filename
#-----------------------------------------------------------------------
LINK_CMD  = --script $(LINKERSCRIPT) --gc-sections --emit-relocs -o #--verbose --no-warn-mismatch
LIBS		= -lSophiaFrameworkBrew400JpnGNUARM41 -lgcc

#-----------------------------------------------------------------------
# HEXTOOL Options
#-----------------------------------------------------------------------
BINFORMAT = -O binary

#-----------------------------------------------------------------------
# Compiler flag definitions
#-----------------------------------------------------------------------
CFLAGS0 =	$(OUT) $(DYNAPP) $(CPU) $(APCS) $(CODE) $(DBG)
CFLAGS =	$(CFLAGS0) $(INC) $(OPT) 

#-----------------------------------------------------------------------
# Linker flag definitions
#-----------------------------------------------------------------------
LDFLAGS =	$(LIBDIRS)

#-----------------------------------------------------------------------
# Default target
#-----------------------------------------------------------------------
default: $(TARGET).$(MODULE)

#-----------------------------------------------------------------------
# All target
#-----------------------------------------------------------------------
all: $(TARGET).$(MODULE)


#-----------------------------------------------------------------------
# C Code inference rules (plus a few others, for debugging purposes)
#   "make foo.s" will compile foo.c into assembly code.
#   "make foo.sp" will produce a foo.s file with the information on
#     which pattern in the RTL was matched to produce each assembly
#     instruction 
#   "make foo.dump" will make foo.o if necessary and then use objdump -D
#     on it, piping the results into foo.dump.  This is slightly
#     different from the output of "make foo.s", in general.
#-----------------------------------------------------------------------
%.o: %.cpp
	$(GCC) $(CFLAGS) -o $@ $< 

OBJ_CMD    = -o               # Command line option to specify output filename
SRC_FILE = $(@F:.o=.cpp)      # Input source file specification
OBJ_FILE = $(OBJ_CMD) $(@F)   # Output object file specification

.SUFFIXES :
.SUFFIXES : .o .dep .cpp

.cpp.o:
	@echo ---------------------------------------------------------------
	@echo OBJECT $(@F)
	$(GCC) $(CFLAGS) $(OBJ_FILE) $(SRC_FILE)
	@echo ---------------------------------------------------------------

#===============================================================================
#                           MODULE SPECIFIC RULES
#===============================================================================

APP_OBJS = $(OBJS) 

#-----------------------------------------------------------------------
# Targets for making the actual binary
#-----------------------------------------------------------------------
$(TARGET).$(MODULE) :   $(TARGET).$(EXETYPE)
	$(ELF2MODTOOL) $(TARGET).$(EXETYPE) $(TARGET).$(MODULE)

$(TARGET).$(EXETYPE) :  $(APP_OBJS)
	$(LD) $(LINK_CMD) $(TARGET).$(EXETYPE) $(LDFLAGS) $(APP_OBJS) $(LIBS)

#-----------------------------------------------------------------------
# Cleanup 
#-----------------------------------------------------------------------
clean:
	rm -f $(APP_OBJS) $(TARGET).$(EXETYPE) $(TARGET).$(MODULE)		

#-----------------------------------------------------------------------
# Object File Dependencies
#   You may well want to add more dependencies here.
#-----------------------------------------------------------------------

#----------------------------------------------------------------------------
# Applet Specific Rules
#----------------------------------------------------------------------------


# --------------------------------------------
# DEPENDENCY LIST, DO NOT EDIT BELOW THIS LINE
# --------------------------------------------


yhelloworld.o:	yhelloworld.cpp
yhelloworld.o:	yhelloworld.hpp
yhelloworld.o:	yhelloworld.bid
